from .client import MB_URL as MB_URL
from .client import MB_VERSION_ID as MB_VERSION_ID
from .client import make_request as make_request
from .client import set_admin_id as set_admin_id
from .client import set_timeout as set_timeout
from .client import set_token as set_token
from .contact import Contact as Contact
from .contact import ContactPerson as ContactPerson
from .external_sales_invoice import ExternalSalesInvoice as ExternalSalesInvoice
from .external_sales_invoice import (
    ExternalSalesInvoiceDetailsAttribute as ExternalSalesInvoiceDetailsAttribute,
)
from .payment import Payment as Payment
from .financial_mutation import FinancialMutation as FinancialMutation
from .financial_statement import FinancialStatement as FinancialStatement
from .tax_rate import TaxRate as TaxRate

__all__ = [
    "MB_URL",
    "MB_VERSION_ID",
    "make_request",
    "set_admin_id",
    "set_timeout",
    "set_token",
    "Contact",
    "ContactPerson",
    "ExternalSalesInvoice",
    "ExternalSalesInvoiceDetailsAttribute",
    "Payment",
    "FinancialMutation",
    "FinancialStatement",
    "TaxRate",
]



from .client import http_delete as http_delete, http_get as http_get, http_patch as http_patch, http_post as http_post
from _typeshed import Incomplete
from pydantic import BaseModel
from typing import Any, Self

class MoneybirdModel(BaseModel):
    id: int | None
    model_config: Incomplete
    @property
    def endpoint(self) -> str: ...
    def to_dict(self) -> dict[str, Any]: ...
    def load(self, id: int) -> None: ...
    def save(self) -> None: ...
    def update(self, data: dict[str, Any]) -> None: ...
    def delete(self) -> None: ...
    @classmethod
    def find_by_id(cls, id: int) -> Self: ...
    @classmethod
    def update_by_id(cls, id: int, data: dict[str, Any]) -> Self: ...
    @classmethod
    def delete_by_id(cls, id: int) -> Self: ...



from typing import Any
from pydantic import BaseModel, field_validator
from .client import http_get
from .custom_field_model import CustomFieldModel


class ContactPerson(BaseModel):
    firstname: str | None = None
    lastname: str | None = None


class Contact(CustomFieldModel):
    company_name: str | None = None
    address1: str | None = None
    address2: str | None = None
    zipcode: str | None = None
    city: str | None = None
    country: str | None = None
    phone: str | None = None
    delivery_method: str | None = None
    customer_id: str | None = None
    tax_number: str | None = None
    firstname: str | None = None
    lastname: str | None = None
    chamber_of_commerce: str | None = None
    bank_account: str | None = None
    send_invoices_to_attention: str | None = None
    send_invoices_to_email: str | None = None
    send_estimates_to_attention: str | None = None
    send_estimates_to_email: str | None = None
    sepa_active: bool = False
    sepa_iban: str | None = None
    sepa_iban_account_name: str | None = None
    sepa_bic: str | None = None
    sepa_mandate_id: str | None = None
    sepa_mandate_date: str | None = None
    sepa_sequence_type: str | None = None
    si_identifier_type: str | None = None
    si_identifier: str | None = None
    invoice_workflow_id: int | None = None
    estimate_workflow_id: int | None = None
    email_ubl: bool = False
    direct_debit: bool = False
    contact_people: list[ContactPerson] | None = None
    type: str | None = None
    from_checkout: bool = False

    @staticmethod
    def find_by_customer_id(customer_id: str) -> "Contact":
        """
        Find a contact by customer_id
        """
        data = http_get(f"contacts/customer_id/{customer_id}")
        return Contact(**data)

    @field_validator("contact_people")
    def ensure_contact_people(
        cls, value: list[dict[str, Any] | ContactPerson] | None
    ) -> list[ContactPerson] | None:
        if value is None:
            return None

        contact_people: list[ContactPerson] = []

        for person in value:
            if isinstance(person, ContactPerson):
                contact_people.append(person)
            else:
                contact_people.append(ContactPerson(**person))

        return contact_people



from .client import http_get as http_get
from .custom_field_model import CustomFieldModel as CustomFieldModel
from pydantic import BaseModel
from typing import Any

class ContactPerson(BaseModel):
    firstname: str | None
    lastname: str | None

class Contact(CustomFieldModel):
    company_name: str | None
    address1: str | None
    address2: str | None
    zipcode: str | None
    city: str | None
    country: str | None
    phone: str | None
    delivery_method: str | None
    customer_id: str | None
    tax_number: str | None
    firstname: str | None
    lastname: str | None
    chamber_of_commerce: str | None
    bank_account: str | None
    send_invoices_to_attention: str | None
    send_invoices_to_email: str | None
    send_estimates_to_attention: str | None
    send_estimates_to_email: str | None
    sepa_active: bool
    sepa_iban: str | None
    sepa_iban_account_name: str | None
    sepa_bic: str | None
    sepa_mandate_id: str | None
    sepa_mandate_date: str | None
    sepa_sequence_type: str | None
    si_identifier_type: str | None
    si_identifier: str | None
    invoice_workflow_id: int | None
    estimate_workflow_id: int | None
    email_ubl: bool
    direct_debit: bool
    contact_people: list[ContactPerson] | None
    type: str | None
    from_checkout: bool
    @staticmethod
    def find_by_customer_id(customer_id: str) -> Contact: ...
    def ensure_contact_people(cls, value: list[dict[str, Any] | ContactPerson] | None) -> list[ContactPerson] | None: ...



from typing import Any

MB_URL: str
MB_VERSION_ID: str
admin_id_: int
token_: str
timeout_: int

def set_admin_id(admin_id: int) -> None: ...
def set_token(token: str) -> None: ...
def set_timeout(timeout: int) -> None: ...
def make_request(path: str, data: dict[str, Any] | None = None, method: str = 'post', params: dict[str, Any] | None = None) -> Any: ...
def http_get(path: str, params: dict[str, Any] | None = None) -> Any: ...
def http_post(path: str, data: dict[str, Any] | None = None) -> Any: ...
def http_patch(path: str, data: dict[str, Any] | None = None) -> Any: ...
def http_delete(path: str) -> Any: ...



from .financial_mutation import FinancialMutation as FinancialMutation
from .model import MoneybirdModel as MoneybirdModel
from moneysnake.client import http_patch as http_patch, http_post as http_post
from typing import Any, Self

class FinancialStatement(MoneybirdModel):
    financial_account_id: str | None
    reference: str | None
    official_date: str | None
    official_balance: str | None
    importer_service: str | None
    financial_mutations: list[FinancialMutation]
    def ensure_financial_mutations(cls, value: list[dict[str, Any]] | None) -> list[FinancialMutation] | None: ...
    def save(self) -> None: ...
    def load(self, id: int) -> None: ...
    def add_financial_mutation(self, financial_mutation: FinancialMutation) -> None: ...
    @classmethod
    def find_by_id(cls, id: int) -> Self: ...



from typing import Any, Self
from pydantic import BaseModel, ConfigDict

from .client import http_delete, http_get, http_patch, http_post


class MoneybirdModel(BaseModel):
    id: int | None = None
    model_config = ConfigDict(extra="ignore")

    @property
    def endpoint(self) -> str:
        return "".join(
            [
                "_" + letter.lower() if letter.isupper() else letter
                for letter in self.__class__.__name__
            ]
        ).lstrip("_")

    def to_dict(self) -> dict[str, Any]:
        return self.model_dump(exclude_none=True)

    def load(self, id: int) -> None:
        data = http_get(f"{self.endpoint}s/{id}")
        self.update(data)

    def save(self) -> None:
        if self.id is None:
            data = http_post(
                f"{self.endpoint}s",
                data={self.endpoint: self.to_dict()},
            )
            # update the current object with the data
            self.update(data)
        else:
            data = http_patch(
                f"{self.endpoint}s/{self.id}",
                data={self.endpoint: self.to_dict()},
            )
            self.update(data)

    def update(self, data: dict[str, Any]) -> None:
        # Use model_validate to ensure field validators run
        validated = self.model_validate({**self.model_dump(), **data})
        # Copy validated field values directly (not via model_dump which converts to dicts)
        for key in self.model_fields:
            object.__setattr__(self, key, getattr(validated, key))

    def delete(self) -> None:
        if not self.id:
            raise ValueError(f"Cannot delete {self.__class__.__name__} without an id")
        http_delete(f"{self.endpoint}s/{self.id}")
        # remove the id from the object
        self.id = None

    @classmethod
    def find_by_id(cls: type[Self], id: int) -> Self:
        entity = cls(id=id)
        entity.load(id)
        return entity

    @classmethod
    def update_by_id(cls: type[Self], id: int, data: dict[str, Any]) -> Self:
        entity = cls(id=id)
        entity.update(data)
        entity.save()
        return entity

    @classmethod
    def delete_by_id(cls: type[Self], id: int) -> Self:
        entity = cls(id=id)
        entity.delete()
        return entity



from .client import http_get as http_get, http_patch as http_patch
from .model import MoneybirdModel as MoneybirdModel
from enum import Enum
from typing import Any, Self

class LinkBookingType(Enum):
    LedgerAccount = ...
    SalesInvoice = ...
    Document = ...
    PaymentTransactionBatch = ...
    PurchaseTransaction = ...
    NewPurchaseInvoice = ...
    NewReceipt = ...
    PaymentTransaction = ...
    PurchaseTransactionBatch = ...
    ExternalSalesInvoice = ...
    Payment = ...
    VatDocument = ...

class UnlinkBookingType(Enum):
    LedgerAccountBooking = ...
    Payment = ...

class FinancialMutation(MoneybirdModel):
    id: int | None
    administration_id: str | int | None
    amount: str | None
    code: str | None
    date: str | None
    message: str | None
    contra_account_name: str | None
    contra_account_number: str | None
    state: str | None
    amount_open: str | None
    sepa_fields: dict | None
    batch_reference: str | None
    financial_account_id: str | None
    currency: str | None
    original_amount: str | None
    created_at: str | None
    updated_at: str | None
    version: str | int | None
    financial_statement_id: str | None
    processed_at: str | None
    account_servicer_transaction_id: str | None
    payments: list[Any]
    ledger_account_bookings: list[Any]
    def save(self) -> None: ...
    def delete(self) -> None: ...
    def book_payment(self, price_base: float, booking_id: int, booking_type: LinkBookingType = ...) -> None: ...
    def remove_payment(self, booking_id: int, booking_type: UnlinkBookingType = ...) -> None: ...
    @classmethod
    def search(cls, query_string: str | None = None, period: str | None = ..., financial_account_id: str | None = None) -> list[Self]: ...
    @classmethod
    def update_by_id(cls, id: int, data: dict[str, Any]) -> Self: ...
    @classmethod
    def delete_by_id(cls, id: int) -> Self: ...



import re
from enum import Enum, auto
from typing import Any, Self
from datetime import datetime
from pydantic import Field

from .model import MoneybirdModel
from .client import http_patch, http_get


class LinkBookingType(Enum):
    """
    Enum for the different types of bookings that can be linked to a financial mutation.
    """

    LedgerAccount = auto()
    SalesInvoice = auto()
    Document = auto()
    PaymentTransactionBatch = auto()
    PurchaseTransaction = auto()
    NewPurchaseInvoice = auto()
    NewReceipt = auto()
    PaymentTransaction = auto()
    PurchaseTransactionBatch = auto()
    ExternalSalesInvoice = auto()
    Payment = auto()
    VatDocument = auto()


class UnlinkBookingType(Enum):
    """
    Enum for the different types of bookings that can be unlinked from a financial mutation.
    """

    LedgerAccountBooking = auto()
    Payment = auto()


class FinancialMutation(MoneybirdModel):
    """
    Represents a financial mutation in Moneybird.
    """

    id: int | None = None
    administration_id: str | int | None = None
    amount: str | None = None
    code: str | None = None
    date: str | None = None
    message: str | None = None
    contra_account_name: str | None = None
    contra_account_number: str | None = None
    state: str | None = None
    amount_open: str | None = None
    sepa_fields: dict | None = None
    batch_reference: str | None = None
    financial_account_id: str | None = None
    currency: str | None = None
    original_amount: str | None = None
    created_at: str | None = None
    updated_at: str | None = None
    version: str | int | None = None
    financial_statement_id: str | None = None
    processed_at: str | None = None
    account_servicer_transaction_id: str | None = None
    payments: list[Any] = Field(default_factory=list)
    ledger_account_bookings: list[Any] = Field(default_factory=list)

    # Disable create, update and delete methods for financial mutations as they don't
    # exist in the Moneybird API.
    def save(self) -> None:
        raise NotImplementedError("Financial mutations cannot be saved in Moneybird.")

    def delete(self) -> None:
        raise NotImplementedError("Financial mutations cannot be deleted in Moneybird.")

    def book_payment(
        self,
        price_base: float,
        booking_id: int,
        booking_type: LinkBookingType = LinkBookingType.LedgerAccount,
    ) -> None:
        """
        Book a payment on this financial mutation.
        """
        http_patch(
            f"financial_mutations/{self.id}/link_booking",
            {
                "price_base": price_base,
                "booking_id": booking_id,
                "booking_type": booking_type.name,
            },
        )

    def remove_payment(
        self,
        booking_id: int,
        booking_type: UnlinkBookingType = UnlinkBookingType.LedgerAccountBooking,
    ) -> None:
        """
        Remove a payment from this financial mutation.
        """
        http_patch(
            f"financial_mutations/{self.id}/unlink_booking",
            {"booking_id": booking_id, "booking_type": booking_type.name},
        )

    @classmethod
    def search(
        cls,
        query_string: str | None = None,
        period: str | None = datetime.now().strftime("%Y%m%d"),
        financial_account_id: str | None = None,
    ) -> list[Self]:
        """
        Search for financial mutations using the standard list endpoint. The query_string
        should be in the format 'key:value,key:value' (e.g., 'state:open,amount:>100').
        If the period is a single day (YYYYMMDD), it will be formatted as a range.
        A specific 'period' is required to avoid the 'Too many mutations' error.
        If no period is provided, it defaults to the current day.
        """
        formatted_period = (
            f"{period}..{period}" if len(period) == 8 and ".." not in period else period
        )
        filter_parts = [f"period:{formatted_period}"]

        if query_string and re.match(
            r"^[\w]+:[\w\s\-\.]+(?:,[\w]+:[\w\s\-\.]+)*$", query_string
        ):
            filter_parts.insert(0, query_string)

        if financial_account_id:
            filter_parts.append(f"financial_account_id:{financial_account_id}")

        filter_val = ",".join(filter_parts)

        params = {"filter": filter_val}

        results = http_get("financial_mutations", params=params)
        return [cls(**result) for result in results]

    @classmethod
    def update_by_id(cls: type[Self], id: int, data: dict[str, Any]) -> Self:
        raise NotImplementedError("Financial mutations cannot be updated in Moneybird.")

    @classmethod
    def delete_by_id(cls: type[Self], id: int) -> Self:
        raise NotImplementedError("Financial mutations cannot be deleted in Moneybird.")



from typing import Any, Self
from pydantic import Field, field_validator

from moneysnake.client import http_patch, http_post

from .model import MoneybirdModel
from .financial_mutation import FinancialMutation


class FinancialStatement(MoneybirdModel):
    """
    Represents a financial statement in Moneybird.
    """

    financial_account_id: str | None = None
    reference: str | None = None
    official_date: str | None = None
    official_balance: str | None = None
    importer_service: str | None = None
    financial_mutations: list[FinancialMutation] = Field(default_factory=list)

    @field_validator("financial_mutations")
    def ensure_financial_mutations(
        cls, value: list[dict[str, Any]] | None
    ) -> list[FinancialMutation] | None:
        if value is None:
            return None

        financial_mutations: list[FinancialMutation] = []

        for financial_mutation in value:
            if isinstance(financial_mutation, FinancialMutation):
                financial_mutations.append(financial_mutation)
            else:
                financial_mutations.append(FinancialMutation(**financial_mutation))

        return financial_mutations

    def save(self) -> None:
        """
        Save the external sales invoice. Overrides the save method in MoneybirdModel.
        """
        financial_statement_data = self.to_dict()

        # For the POST and PATCH requests we need to use the details_attributes key
        # instead of details key to match the Moneybird API.
        financial_statement_data["financial_mutations_attributes"] = (
            financial_statement_data.pop("financial_mutations", [])
        )

        if self.id is None:
            data = http_post(
                f"{self.endpoint}s",
                data={self.endpoint: financial_statement_data},
            )

        else:
            data = http_patch(
                f"{self.endpoint}s/{self.id}",
                data={self.endpoint: financial_statement_data},
            )
        self.update(data)

    def load(self, id: int) -> None:
        raise NotImplementedError(
            "Financial statements cannot be loaded from Moneybird."
        )

    def add_financial_mutation(self, financial_mutation: FinancialMutation) -> None:
        """
        Add a financial mutation to the financial statement.
        """
        self.financial_mutations.append(financial_mutation)

    @classmethod
    def find_by_id(cls: type[Self], id: int) -> Self:
        raise NotImplementedError(
            "Financial statements cannot be loaded from Moneybird."
        )



from .client import http_get as http_get
from .model import MoneybirdModel as MoneybirdModel
from typing import Self

class TaxRate(MoneybirdModel):
    name: str | None
    percentage: float | None
    tax_rate_type: str | None
    country: str | None
    active: bool | None
    administration_id: int | None
    show_tax: bool | None
    created_at: str | None
    updated_at: str | None
    @classmethod
    def list_all_rates(cls) -> list[Self]: ...
    @classmethod
    def list_sales_rates(cls) -> list[Self]: ...
    @classmethod
    def find_sales_rate_by_country(cls, country: str) -> list[Self]: ...



from .model import MoneybirdModel as MoneybirdModel
from pydantic import BaseModel

class CustomField(BaseModel):
    id: int | None
    value: str | None

class CustomFieldModel(MoneybirdModel):
    custom_fields: list[CustomField]
    def get_custom_field(self, field_id: int) -> str | None: ...
    def set_custom_field(self, field_id: int, value: str) -> None: ...



from pydantic import Field, field_validator
from typing import Any, cast

from pydantic import BaseModel

from .model import MoneybirdModel
from .client import http_delete, http_get, http_patch, http_post
from .payment import Payment


class ExternalSalesInvoiceDetailsAttribute(BaseModel):
    """
    Details attribute for an external sales invoice.
    """

    id: int | None = None
    description: str | None = None
    period: str | None = None
    price: int | str | None = None
    amount: int | str | None = None
    tax_rate_id: int | None = None
    ledger_account_id: str | None = None
    project_id: str | None = None

    def update(self, data: dict[str, Any]) -> None:
        for key, value in data.items():
            if hasattr(self, key):
                setattr(self, key, value)


# External sales invoices don't have custom fields, so we use MoneybirdModel
# instead of CustomFieldModel.


class ExternalSalesInvoice(MoneybirdModel):
    """
    Represents an external sales invoice in Moneybird.
    """

    contact_id: int | None = None
    reference: str | None = None
    date: str | None = None
    due_date: str | None = None
    currency: str | None = None
    prices_are_incl_tax: bool | None = None
    source: str | None = None
    source_url: str | None = None
    details: list[ExternalSalesInvoiceDetailsAttribute] | None = Field(
        default_factory=list
    )
    payments: list[Payment] | None = Field(default_factory=list)

    @field_validator("payments")
    def ensure_payments(
        cls, value: list[dict[str, Any]] | list[Payment] | None
    ) -> list[Payment] | None:
        if value is None:
            return None

        return [p if isinstance(p, Payment) else Payment(**p) for p in value]

    def update(self, data: dict[str, Any]) -> None:
        """
        Update the external sales invoice. Overrides the update method in MoneybirdModel.
        """
        super().update(data)
        if self.payments is not None:
            pass
            # self.payments = [
            #     Payment.model_dump(p) if isinstance(p, dict) else p for p in self.payments
            # ]

    def save(self) -> None:
        """
        Save the external sales invoice. Overrides the save method in MoneybirdModel.
        """
        invoice_data = self.to_dict()
        # For the POST and PATCH requests we need to use the details_attributes key
        # instead of details key to match the Moneybird API.
        invoice_data["details_attributes"] = invoice_data.pop("details", [])

        if self.id is None:
            data = http_post(
                f"{self.endpoint}s",
                data={self.endpoint: invoice_data},
            )
        else:
            data = http_patch(
                f"{self.endpoint}s/{self.id}",
                data={self.endpoint: invoice_data},
            )
        self.update(data)

    def add_detail(self, detail: ExternalSalesInvoiceDetailsAttribute) -> None:
        """
        Add a detail to the external sales invoice.
        """
        if self.details is None:
            self.details = []

        self.details.append(detail)

    def get_detail(self, detail_id: int) -> ExternalSalesInvoiceDetailsAttribute:
        """
        Get a detail from the external sales invoice.
        """

        if not self.details:
            raise ValueError("No details found.")

        for detail in self.details:
            if detail.id == detail_id:
                return detail
        raise ValueError(f"Detail with id {detail_id} not found.")

    def update_detail(
        self, detail_id: int, data: dict[str, Any]
    ) -> ExternalSalesInvoiceDetailsAttribute:
        """
        Update a detail from the external sales invoice.
        """
        detail = self.get_detail(detail_id)
        detail.update(data)
        return detail

    def delete_detail(self, detail_id: int) -> None:
        """
        Delete a detail from the external sales invoice.
        """

        if self.details:
            self.details = [detail for detail in self.details if detail.id != detail_id]

    def list_all_by_contact_id(
        self,
        contact_id: int,
        state: str | None = "all",
        period: str | None = "this_year",
    ) -> list["ExternalSalesInvoice"]:
        """
        List all external sales invoices for a contact.
        """
        data = http_get(
            path=f"{self.endpoint}s/?filter=contact_id:{contact_id}&state:{state}&period:{period}"
        )

        data_list = cast(list[dict[str, Any]], data)

        invoices: list[ExternalSalesInvoice] = []
        for invoice_data in data_list:
            invoice_obj = ExternalSalesInvoice(**invoice_data)
            invoices.append(invoice_obj)
        return invoices

    def create_payment(self, payment: Payment) -> None:
        """
        Create a payment for the external sales invoice.
        """
        data = http_post(
            path=f"{self.endpoint}s/{self.id}/payments",
            data={"payment": payment.to_dict()},
        )
        # Get the payment data from the response and append it to the payments list
        payment_data = data.get("payment")

        if self.payments is None:
            self.payments = []

        if payment_data:
            self.payments.append(Payment(**payment_data))

    def delete_payment(self, payment_id: int) -> None:
        """
        Delete a payment for the external sales invoice.
        """
        http_delete(
            path=f"{self.endpoint}s/{self.id}/payments/{payment_id}",
        )

        if not self.payments:
            raise ValueError("No payments found.")

        self.payments = [
            payment for payment in self.payments if payment.id != payment_id
        ]



from .model import MoneybirdModel


class Payment(MoneybirdModel):
    """
    Represents a payment in Moneybird.
    """

    payment_date: str | None = None
    price: float | None = None
    price_base: float | None = None
    financial_account_id: int | None = None
    financial_mutation_id: int | None = None
    manual_payment_action: str | None = "bank_transfer"
    transaction_identifier: str | None = None
    ledger_account_id: int | None = None
    invoice_id: int | None = None

    def save(self) -> None:
        raise NotImplementedError(
            "Payments cannot be saved directly. Refer to the invoice that the payment is for."
        )

    def delete(self) -> None:
        raise NotImplementedError(
            "Payments cannot be deleted directly. Refer to the invoice that the payment is for."
        )



from .client import http_delete as http_delete, http_get as http_get, http_patch as http_patch, http_post as http_post
from .model import MoneybirdModel as MoneybirdModel
from .payment import Payment as Payment
from pydantic import BaseModel
from typing import Any

class ExternalSalesInvoiceDetailsAttribute(BaseModel):
    id: int | None
    description: str | None
    period: str | None
    price: int | str | None
    amount: int | str | None
    tax_rate_id: int | None
    ledger_account_id: str | None
    project_id: str | None
    def update(self, data: dict[str, Any]) -> None: ...

class ExternalSalesInvoice(MoneybirdModel):
    contact_id: int | None
    reference: str | None
    date: str | None
    due_date: str | None
    currency: str | None
    prices_are_incl_tax: bool | None
    source: str | None
    source_url: str | None
    details: list[ExternalSalesInvoiceDetailsAttribute] | None
    payments: list[Payment] | None
    def ensure_payments(cls, value: list[dict[str, Any]] | list[Payment] | None) -> list[Payment] | None: ...
    def update(self, data: dict[str, Any]) -> None: ...
    def save(self) -> None: ...
    def add_detail(self, detail: ExternalSalesInvoiceDetailsAttribute) -> None: ...
    def get_detail(self, detail_id: int) -> ExternalSalesInvoiceDetailsAttribute: ...
    def update_detail(self, detail_id: int, data: dict[str, Any]) -> ExternalSalesInvoiceDetailsAttribute: ...
    def delete_detail(self, detail_id: int) -> None: ...
    def list_all_by_contact_id(self, contact_id: int, state: str | None = 'all', period: str | None = 'this_year') -> list['ExternalSalesInvoice']: ...
    def create_payment(self, payment: Payment) -> None: ...
    def delete_payment(self, payment_id: int) -> None: ...



from typing import Self

from .client import http_get
from .model import MoneybirdModel


class TaxRate(MoneybirdModel):
    """
    Represents a Tax Rate in Moneybird.
    """

    name: str | None = None
    percentage: float | None = None
    tax_rate_type: str | None = None
    country: str | None = None
    active: bool | None = None
    administration_id: int | None = None
    show_tax: bool | None = None
    created_at: str | None = None
    updated_at: str | None = None

    @classmethod
    def list_all_rates(cls) -> list[Self]:
        """
        List all available tax rates for the administration.
        """
        data = http_get("tax_rates")
        return [cls(**rate) for rate in data]

    @classmethod
    def list_sales_rates(cls) -> list[Self]:
        """
        List all sales tax rates.
        """
        data = http_get("tax_rates?filter=tax_rate_type:sales_invoice")
        return [cls(**rate) for rate in data]

    @classmethod
    def find_sales_rate_by_country(cls, country: str) -> list[Self]:
        """
        Find sales tax rates by country.
        """
        data = http_get(
            f"tax_rates?filter=country:{country},tax_rate_type:sales_invoice"
        )
        return [cls(**rate) for rate in data]



from .model import MoneybirdModel as MoneybirdModel

class Payment(MoneybirdModel):
    payment_date: str | None
    price: float | None
    price_base: float | None
    financial_account_id: int | None
    financial_mutation_id: int | None
    manual_payment_action: str | None
    transaction_identifier: str | None
    ledger_account_id: int | None
    invoice_id: int | None
    def save(self) -> None: ...
    def delete(self) -> None: ...



from typing import Any

import httpx

MB_URL = "https://moneybird.com/api"
MB_VERSION_ID = "v2"

# Moneybird settings
admin_id_ = 0
token_ = ""
timeout_ = 20


def set_admin_id(admin_id: int) -> None:
    global admin_id_
    admin_id_ = admin_id


def set_token(token: str) -> None:
    global token_
    token_ = token


def set_timeout(timeout: int) -> None:
    global timeout_
    timeout_ = timeout


def make_request(
    path: str,
    data: dict[str, Any] | None = None,
    method: str = "post",
    params: dict[str, Any] | None = None,
) -> Any:
    headers = {
        "Authorization": f"Bearer {token_}",
        "Content-Type": "application/json",
    }
    fullpath = f"{MB_URL}/{MB_VERSION_ID}/{admin_id_}/{path}"
    response = httpx.request(
        method,
        fullpath,
        json=data,
        headers=headers,
        timeout=timeout_,
        params=params,
    )
    response.raise_for_status()

    # return json if there is content
    return response.json() if response.content else {}


def http_get(path: str, params: dict[str, Any] | None = None) -> Any:
    return make_request(path, method="get", params=params)


def http_post(path: str, data: dict[str, Any] | None = None) -> Any:
    return make_request(path, method="post", data=data)


def http_patch(path: str, data: dict[str, Any] | None = None) -> Any:
    return make_request(path, method="patch", data=data)


def http_delete(path: str) -> Any:
    return make_request(path, method="delete")
